'use strict';

import {
    NativeModules,
    NativeEventEmitter,
} from 'react-native';

const { Appodeal } = NativeModules;

const NONE = 0;
const INTERSTITIAL = 3;
const BANNER = 4;
const BANNER_BOTTOM = 8;
const BANNER_TOP = 16;
const BANNER_VIEW = 64;
const REWARDED_VIDEO = 128;
const NON_SKIPPABLE_VIDEO = 128;
const NATIVE = 512;

const eventEmitter = new NativeEventEmitter(Appodeal);

const eventHandlers = {
    //interstitial
    onInterstitialLoaded: 'onInterstitialLoaded',
    onInterstitialFailedToLoad: 'onInterstitialFailedToLoad',
    onInterstitialShown: 'onInterstitialShown',
    onInterstitialShowFailed: 'onInterstitialShowFailed',
    onInterstitialClicked: 'onInterstitialClicked',
    onInterstitialClosed: 'onInterstitialClosed',
    onInterstitialExpired: 'onInterstitialExpired',
    
    //rewarded video
    onRewardedVideoLoaded: 'onRewardedVideoLoaded',
    onRewardedVideoFailedToLoad: 'onRewardedVideoFailedToLoad',
    onRewardedVideoShown: 'onRewardedVideoShown',
    onRewardedVideoShowFailed: 'onRewardedVideoShowFailed',
    onRewardedVideoFinished: 'onRewardedVideoFinished',
    onRewardedVideoClosed: 'onRewardedVideoClosed',
    onRewardedVideoExpired: 'onRewardedVideoExpired',
    onRewardedVideoClicked: 'onRewardedVideoClicked',
    
    //non skippable video
    onNonSkippableVideoClosed: 'onNonSkippableVideoClosed',
    onNonSkippableVideoFailedToLoad: 'onNonSkippableVideoFailedToLoad',
    onNonSkippableVideoShown: 'onNonSkippableVideoShown',
    onNonSkippableVideoShowFailed: 'onNonSkippableVideoShowFailed',
    onNonSkippableVideoFinished: 'onNonSkippableVideoFinished',
    onNonSkippableVideoLoaded: 'onNonSkippableVideoLoaded',
    onNonSkippableVideoExpired: 'onNonSkippableVideoExpired',

    //native
    onNativeLoaded: 'onNativeLoaded',
    onNativeFailedToLoad: 'onNativeFailedToLoad',
    onNativeShown: 'onNativeShown',
    onNativeShowFailed: 'onNativeShowFailed',
    onNativeClicked: 'onNativeClicked',
    onNativeExpired: 'onNativeExpired',
    
    //banner
    onBannerLoaded: 'onBannerLoaded',
    onBannerFailedToLoad: 'onBannerFailedToLoad',
    onBannerShown: 'onBannerShown',
    onBannerShowFailed: 'onBannerShowFailed',
    onBannerClicked: 'onBannerClicked',
    onBannerExpired: 'onBannerExpired'
};

const LogLevel = {
    none: 'none',
    debug: 'debug',
    verbose: 'verbose'
}

const Gender = {
    male: 'male',
    female: 'female',
    other: 'other'
}

const _subscriptions = new Map();

const addEventListener = (event, handler) => {
    const mappedEvent = eventHandlers[event];
    if (mappedEvent) {
        let listener;
        listener = eventEmitter.addListener(mappedEvent, handler);
        _subscriptions.set(handler, listener);
        return {
            remove: () => removeEventListener(event, handler)
        };
    } else {
        console.warn(`Trying to subscribe to unknown event: "${event}"`);
        return {
            remove: () => {},
        };
    }
};

const removeEventListener = (type, handler) => {
    const listener = _subscriptions.get(handler);
    if (!listener) {
        return;
    }
    listener.remove();
    _subscriptions.delete(handler);
};

const removeAllListeners = () => {
    _subscriptions.forEach((listener, key, map) => {
        listener.remove();
        map.delete(key);
    });
};

module.exports = {
    ...Appodeal,
    NONE,
    INTERSTITIAL,
    BANNER,
    BANNER_TOP,
    BANNER_VIEW,
    BANNER_BOTTOM,
    REWARDED_VIDEO,
    NON_SKIPPABLE_VIDEO,
    NATIVE,
    LogLevel,
    Gender,
    addEventListener,
    removeEventListener,
    removeAllListeners
};